using System;
using System.Threading;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;


namespace Annabooks.MPL115A1
{
    public class MPL115A1
    {
        private SpiDevice sensorMPL115A1;
        private double a0;
        private double b1;
        private double b2;
        private double c12;

        private bool MPL115A1Ready = false;
        public string connectionError = "";

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MPL115A1(string SPI_Port, int frequency, int SPI_Chip_Select, bool SPI_Bus_Shared)
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            MPL115A1SPISetup(SPI_Port, frequency, SPI_Chip_Select, SPI_Bus_Shared);
        }

        private async void MPL115A1SPISetup(string SPI_Port, int frequency, int SPI_Chip_Select, bool SPI_Bus_Shared)
        {

            try
            {
                SpiConnectionSettings spiSettings = new SpiConnectionSettings(SPI_Chip_Select);
                spiSettings.ClockFrequency = frequency;
                if (SPI_Bus_Shared)
                {
                    spiSettings.SharingMode = SpiSharingMode.Shared;
                }
                else
                {
                    spiSettings.SharingMode = SpiSharingMode.Exclusive;
                }
                spiSettings.Mode = SpiMode.Mode0;

                string aqs = SpiDevice.GetDeviceSelector(SPI_Port);
                DeviceInformationCollection spiDeviceInfo = await DeviceInformation.FindAllAsync(aqs);
                sensorMPL115A1 = await SpiDevice.FromIdAsync(spiDeviceInfo[0].Id, spiSettings);
            }
            catch (Exception ex)
            {
                connectionError = ex.Message;
            }

            if (sensorMPL115A1 != null)
            {
                MPL115A1Ready = true;
                //we will get the coefficients now since they never change.
                byte[] coeffData = { 0x88, 0x0, 0x8A, 0x0, 0x8C, 0x0, 0x8E, 0x0, 0x90, 0x0, 0x92, 0x0, 0x94, 0x0, 0x96, 0x0, 0x0 };
                byte[] readCOEFF = new byte[17];

                sensorMPL115A1.TransferFullDuplex(coeffData, readCOEFF);

                var coefA = readCOEFF[1] << 8 | readCOEFF[3];
                var coefB1 = readCOEFF[5] << 8 | readCOEFF[7];
                var coefB2 = readCOEFF[9] << 8 | readCOEFF[11];
                var coefC12 = readCOEFF[13] << 8 | readCOEFF[15];

                //convert coeffs
                bool coefAneg = false;
                if ((coefA & 0x8000) == 0x8000)
                {
                    coefA = (coefA ^ 0xFFFF) + 1; //twos complimnet
                    coefAneg = true;
                }

                var coefAInt = coefA >> 3;
                var coefAfrac = coefA & 0x7;
                if (coefAneg)
                {
                    coefAInt = -coefAInt;
                    coefAfrac = -coefAfrac;
                }
                a0 = (double)coefAInt + (double)coefAfrac / 8;

                bool coefB1neg = false;
                if ((coefB1 & 0x8000) == 0x8000)
                {
                    coefB1 = (coefB1 ^ 0xFFFF) + 1;
                    coefB1neg = true;
                }

                var coefB1Int = coefB1 >> 13;
                var coefB1Frac = coefB1 & 0x1FFF;
                if (coefB1neg)
                {
                    coefB1Int = -coefB1Int;
                    coefB1Frac = -coefB1Frac;
                }
                b1 = (double)coefB1Int + (double)coefB1Frac / 8192;

                bool coefB2neg = false;
                if ((coefB2 & 0x8000) == 0x8000)
                {
                    coefB2 = (coefB2 ^ 0xFFFF) + 1;
                    coefB2neg = true;
                }

                var coefB2Int = coefB2 >> 14;
                var coefB2Frac = coefB2 & 0x3FFF;
                if (coefB2neg)
                {
                    coefB2Int = -coefB2Int;
                    coefB2Frac = -coefB2Frac;
                }
                b2 = (double)coefB2Int + (double)coefB2Frac / 16384;

                bool coefC12neg = false;
                if ((coefC12 & 0x8000) == 0x8000)
                {
                    coefC12 = (coefC12 ^ 0xFFFF) + 1;
                    coefC12neg = true;
                }

                var coefC12Frac = coefC12 >> 2;
                if (coefC12neg)
                {
                    coefC12Frac = -coefC12Frac;
                }
                c12 = (double)coefC12Frac / 4194303;
            }
        }

        public double ReadPressure_kPa()
        {
            byte[] starPTconversion = { 0x24, 0x0 };
            byte[] readPTConversionStart = new byte[2];
            byte[] getPT = { 0x80, 0x0, 0x82, 0x0, 0x84, 0x0, 0x86, 0x0, 0x0 };
            byte[] readPT = new byte[9];
            sensorMPL115A1.TransferFullDuplex(starPTconversion, readPTConversionStart); // data returned is always zero
            Thread.Sleep(50);
            sensorMPL115A1.TransferFullDuplex(getPT, readPT);

            var Padc = (readPT[1] << 8 | readPT[3]) >> 6;
            var Tadc = (readPT[5] << 8 | readPT[7]) >> 6;

            double Pcomp = a0 + ((b1 + (c12 * Tadc)) * Padc) + (b2 * Tadc);
            double PressureResult = (Pcomp * 65 / 1023) + 50;
            return Math.Round(PressureResult, 2);
        }

        public double ReadPressure_inHg()
        {
            double PressureinHg = ReadPressure_kPa() * .02953 / 0.1;
            return Math.Round(PressureinHg, 2);
        }

        public bool MPL115A1Status()
        {
            return MPL115A1Ready;
        }

    }
}
